#!/usr/bin/env runhaskell

import           Data.SCAD
import           Data.Shared
import           Data.URDF.Xml     (writeXML)
import           Data.Walker
import           Linear.Quaternion
import           Linear.V3

data LegPos = RL | RR | FR | FL deriving Eq;

toteTipPos = V3 100 0 0
femurOffset = V3 55 0 0
tibiaOffset = V3 70 0 0
legPose :: LegPos -> Pose
legPose pos = Pose (V3 (xval*20) (yval*20) 0) $ axisAngle zAxis ( angle * pi/180 )
    where
        xval = if pos == FR || pos == RR then 1 else -1
        yval = if pos == FL || pos == FR then 1 else -1
        angle = case pos of
            RL -> -90 - 45
            RR -> -90 + 45
            FL -> 90 + 45
            FR -> 90 - 45

tibia = Part {
        model = scadModel "tibia" [ ("tipOffset",toteTipPos) ] ["tibia.scad"],
        collision_models = [
            scadModel "tibia_collision" [("tipOffset", toteTipPos)] ["tibia.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }


femur = Part {
        model = scadModel "femur" [ ( "tibiaOffset", tibiaOffset)]  ["femur.scad"],
        collision_models = [
            scadModel "femur_collision_a" [("tibiaOffset", tibiaOffset)] ["femur.scad"],
            scadModel "femur_collision_b" [("tibiaOffset", tibiaOffset)] ["femur.scad"],
            scadModel "femur_collision_c" [("tibiaOffset", tibiaOffset)] ["femur.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }

coxa = Part {
        model = scadModel "coxa" [ ( "femurOffset", femurOffset)] ["coxa.scad"],
        collision_models = [
            scadModel "coxa_collision" [ ( "femurOffset", femurOffset)] ["coxa.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }

toteBody legs = Body $ Part {
        model = scadModel "body" [("positions", map (pos.pose) legs)] ["body.scad"],
        collision_models = [
            scadModel "body_collision_a" [("positions", map (pos.pose) legs)] ["body.scad"],
            scadModel "body_collision_b" [("positions", map (pos.pose) legs)] ["body.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }


leg :: LegPos -> Leg
leg pos = Leg {
        leg_name = case pos of
            RL -> "rl"
            RR -> "rr"
            FL -> "fl"
            FR -> "fr"
        ,
        chain = [
            Link "coxa"  noOffset    coxa  zAxis (-pi/2) 0 (pi/2) 50 100 8,
            Link "femur" femurOffset femur yAxis (-pi/2) (-pi/8) (pi/2) 50 100 8,
            Link "tibia" tibiaOffset tibia yAxis (-pi) (pi/2) pi 50 100 8
        ],
        tip = Tip {
            point = toteTipPos
        },
        pose = legPose pos
    }

tote :: Walker
tote = Walker walker_name legs (toteBody legs)
    where
        walker_name = "Tote"
        legs = [leg RR, leg RL, leg FR, leg FL]

main :: IO ()
main = do
   mapM_ (uncurry renderSCAD) $ getSCAD tote
   mapM_ (uncurry renderSTL) $ getSTLRenderList tote
   writeXML (getURDF tote) "urdf/tote.urdf"
